import 'package:flutter/material.dart';
import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:tetrisdual/config/default_activity_settings.dart';
import 'package:tetrisdual/models/settings/settings_activity.dart';

part 'settings_activity_state.dart';

class ActivitySettingsCubit extends HydratedCubit<ActivitySettingsState> {
  ActivitySettingsCubit()
      : super(ActivitySettingsState(settings: ActivitySettings.createDefault()));

  void setValues({
    String? gameType,
  }) {
    emit(
      ActivitySettingsState(
        settings: ActivitySettings(
          gameType: gameType ?? state.settings.gameType,
        ),
      ),
    );
  }

  String getParameterValue(String code) {
    switch (code) {
      case DefaultActivitySettings.parameterCodeGameType:
        return ActivitySettings.getGameTypeValueFromUnsafe(state.settings.gameType);
    }

    return '';
  }

  void setParameterValue(String code, String value) {
    final String gameType = code == DefaultActivitySettings.parameterCodeGameType
        ? value
        : getParameterValue(DefaultActivitySettings.parameterCodeGameType);

    setValues(
      gameType: gameType,
    );
  }

  @override
  ActivitySettingsState? fromJson(Map<String, dynamic> json) {
    final String gameType = json[DefaultActivitySettings.parameterCodeGameType] as String;

    return ActivitySettingsState(
      settings: ActivitySettings(
        gameType: gameType,
      ),
    );
  }

  @override
  Map<String, dynamic>? toJson(ActivitySettingsState state) {
    return <String, dynamic>{
      DefaultActivitySettings.parameterCodeGameType: state.settings.gameType,
    };
  }
}
