import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:tetrisdual/models/activity/player.dart';
import 'package:tetrisdual/models/settings/settings_activity.dart';
import 'package:tetrisdual/models/settings/settings_global.dart';

class Activity {
  Activity({
    // Settings
    required this.activitySettings,
    required this.globalSettings,

    // State
    this.isRunning = false,
    this.isStarted = false,
    this.isFinished = false,
    this.animationInProgress = false,

    // Base data
    this.players = const [],

    // Game data
    this.currentPlayer = 0,
  });

  // Settings
  final ActivitySettings activitySettings;
  final GlobalSettings globalSettings;

  // State
  bool isRunning;
  bool isStarted;
  bool isFinished;
  bool animationInProgress;

  // Base data
  List<Player> players;

  // Game data
  int currentPlayer;

  factory Activity.createEmpty() {
    return Activity(
      // Settings
      activitySettings: ActivitySettings.createDefault(),
      globalSettings: GlobalSettings.createDefault(),
    );
  }

  factory Activity.createNew({
    ActivitySettings? activitySettings,
    GlobalSettings? globalSettings,
  }) {
    final ActivitySettings newActivitySettings =
        activitySettings ?? ActivitySettings.createDefault();
    final GlobalSettings newGlobalSettings = globalSettings ?? GlobalSettings.createDefault();

    return Activity(
      // Settings
      activitySettings: newActivitySettings,
      globalSettings: newGlobalSettings,
      // State
      isRunning: true,
      // Base data
      players: [
        Player(1),
        Player(2),
      ],
      // Game data
      currentPlayer: 0,
    );
  }

  bool get canBeResumed => isStarted && !isFinished;

  void dump() {
    printlog('');
    printlog('## Current game dump:');
    printlog('');
    printlog('$Activity:');
    printlog('  Settings');
    activitySettings.dump();
    globalSettings.dump();
    printlog('  State');
    printlog('    isRunning: $isRunning');
    printlog('    isStarted: $isStarted');
    printlog('    isFinished: $isFinished');
    printlog('    animationInProgress: $animationInProgress');
    printlog('  Base data');
    printlog('    players: $players');
    printlog('  Game data');
    printlog('    currentPlayer: $currentPlayer');
    printlog('');
  }

  @override
  String toString() {
    return '$Activity(${toJson()})';
  }

  Map<String, dynamic>? toJson() {
    return <String, dynamic>{
      // Settings
      'activitySettings': activitySettings.toJson(),
      'globalSettings': globalSettings.toJson(),
      // State
      'isRunning': isRunning,
      'isStarted': isStarted,
      'isFinished': isFinished,
      'animationInProgress': animationInProgress,
      // Base data
      'players': players,
      // Game data
      'currentPlayer': currentPlayer,
    };
  }
}
